import pytest

from .. import VariableDoesNotExists
from .common import write_data
from .test_dot_env import DATA
import os
import zeroncy


@pytest.fixture
def prepare_json_file() -> None:
    """
    Write env vars data into .env.json data...
    """

    write_data("json", DATA)
    zeroncy.config()


def test_check_data(prepare_json_file) -> None:
    """
    Values checks
    """

    assert not zeroncy.get("DEBUG", cast=bool)
    assert zeroncy.get("NAME") == "Victor"
    assert zeroncy.get("PATH") == os.environ["PATH"]
    assert isinstance(zeroncy.get("ALLOWED_HOSTS", many=True), list)

    with pytest.raises(VariableDoesNotExists):
        assert zeroncy.get("#FOO")



import pytest

from typing import List
import os

from .common import write_data
from .. import VariableDoesNotExists, FILE_TYPES_AVALIABLE, JsonFileReader
import zeroncy

DATA = {"DEBUG": "False", "NAME": "Victor", "ALLOWED_HOSTS": "localhost, 127.0.0.0.1", "#FOO": "FOO"}


@pytest.fixture
def prepare_dot_env_file() -> None:
    write_data("", DATA)
    with open(os.getcwd() + "/.env", "w") as file:
        for k, v in DATA.items():
            file.write(f"{k}={v}\n")

    zeroncy.config()


def test_env_vars_value(prepare_dot_env_file):
    """
    Test env variables here
    """
    assert not zeroncy.get("DEBUG", cast=bool)
    assert zeroncy.get("NAME") == "Victor"
    assert zeroncy.get("PATH") == os.environ["PATH"]
    assert isinstance(zeroncy.get("ALLOWED_HOSTS", many=True), list)

    with pytest.raises(VariableDoesNotExists):
        assert zeroncy.get("#FOO")



import os


def write_data(file_type: str, data: dict) -> None:
    """
    Writes vars into config file for unit tests
    """

    filename = "/.env.json" if file_type == "json" else "/.env"
    separator = ":" if file_type == "json" else "="

    with open(os.getcwd() + filename, "w") as file:
        if filename == "/.env.json":
            file.write("{")

        for k, v in data.items():
            file.write(f"{k}{separator}{v}\n")

        if filename == "/.env.json":
            file.write("}")



"""
Controller classes to read the content
of configuration file.

The type file can be choosed between .env or .env.json file
"""

from abc import ABC, abstractmethod, abstractproperty
import os
import json


class AbstractEnvironmentFileReader(ABC):
    """
    Config files reader template.

    Contains abstract methods, that should be
    implemented by your childs. The aim of this class
    is define an interface of methods that allows read and
    providing env variables allocated on a determined configuration
    file. The type of config file may vary...

    If file type is None, its intended that file is .env
    """

    @property
    @abstractmethod
    def file_type():
        pass

    @abstractmethod
    def read_file() -> dict:
        pass


class DotEnvFileReader(AbstractEnvironmentFileReader):
    """
    Reads a .env file
    """

    @property
    def file_type(self) -> str:
        return self._file_type

    @file_type.setter
    def file_type(self, attr: str) -> None:
        self._file_type = attr

    def read_file(self) -> dict:
        lines = open(os.getcwd() + "/.env", encoding="utf-8").readlines()
        dotenv_vars = {x[0]: x[1].strip("\n") for x in [v.split("=") for v in lines]}
        dotenv_vars.update(os.environ)
        return dotenv_vars


class JsonFileReader(AbstractEnvironmentFileReader):
    """
    reads a .env.json file
    """

    @property
    def file_type(self) -> str:
        return self.file_type

    @file_type.setter
    def file_type(self, attr: str) -> None:
        self._file_type = attr

    def read_file(self) -> dict:
        dotenv_vars = json.loads(open(os.getcwd() + "/.env.json", encoding="utf-8").read())
        dotenv_vars.update(os.environ)
        return dotenv_vars



"""
______   _____   _____   _____  _      _  _____ __     __
    /   |       |     | |     | |\     | |       \     /
   /    |_____  |_____| |     | | \__  | |        \___/
  /     |       |    |  |     | |    \ | |          | 
 /_____ |_____  |     | |_____| |     \| |_____     |


 Welcome to zeroncy!!!

 A simple zero-dependency python library...
"""

from collections import OrderedDict

from .exceptions import FileExtensionDoesNotCovered, VariableDoesNotExists
from .controllers import DotEnvFileReader, JsonFileReader

FILE_TYPES_AVALIABLE = {None: DotEnvFileReader, dict: JsonFileReader}
ENV = OrderedDict()


def config(file_type: str = None) -> None:
    """
    The method that will call the Reader for
    file type choosed by user and call read_file
    method to make all env variables avaliable to be
    retrieved calling 'get' method

    @param:
        file_type: str -> type of configuration file that
        user writes env vars...
    """

    if not file_type in FILE_TYPES_AVALIABLE.keys():
        raise FileExtensionDoesNotCovered()
    single_controller = FILE_TYPES_AVALIABLE.get(file_type)()
    global ENV
    raw_data = single_controller.read_file()

    # Cleaning commented lines
    for key, value in raw_data.items():
        if not key.startswith("#"):
            ENV.update({key: value})


def get(var_name: str, cast: any = None, many: bool = False) -> any:
    """
    Get a specific env variable located on config file.
    If the var does not in config file, an VariableDoesNotExists
    will be raised...

    @param:
        var_name: str
        cast: any  -> to convert the var value to a detemined type
        many: bool -> if True, return a list of values
    """

    try:
        if cast is not None:
            var = cast(ENV[var_name])
            if cast == bool:
                var = not var
        else:
            var = (
                ENV[var_name].replace(" ", "").split(",")
                if many
                else ENV[var_name]
            )
        return var
    except KeyError:
        raise VariableDoesNotExists()
